#!/usr/bin/env python

import argparse
import datetime
import logging
import os
import time

import sqlalchemy
import pandas as pd

def setup_logging(args, uuid):
    logging.basicConfig(
        filename=os.path.join(uuid + '.log'),
        level=args.level,
        filemode='w',
        format='%(asctime)s %(levelname)s %(message)s',
        datefmt='%Y-%m-%d_%H:%M:%S_%Z',
    )
    logging.getLogger('sqlalchemy.engine').setLevel(logging.INFO)
    logger = logging.getLogger(__name__)
    return logger

def main():
    parser = argparse.ArgumentParser('update status of job')
    # Logging flags.
    parser.add_argument('-d', '--debug',
        action = 'store_const',
        const = logging.DEBUG,
        dest = 'level',
        help = 'Enable debug logging.',
    )
    parser.set_defaults(level = logging.INFO)

    parser.add_argument('--bam_signpost_id',
                        required=True
    )
    parser.add_argument('--bam_uuid',
                        required=False
    )
    parser.add_argument('--hostname',
                        required=True
    )
    parser.add_argument('--host_ipaddress',
                        required=True
    )
    parser.add_argument('--host_mac',
                        required=True
    )
    parser.add_argument('--job_creation_uuid',
                        required=True
    )
    parser.add_argument('--job_path',
                        required=True
    )    
    parser.add_argument('--known_snp_signpost_id',
                        required=True
    )
    parser.add_argument('--num_threads',
                        required=True
    )
    parser.add_argument('--reference_fa_signpost_id',
                        required=True
    )
    parser.add_argument('--run_uuid',
                        required=True
    )
    parser.add_argument('--runner_cwl_path',
                        required=True
    )
    parser.add_argument('--runner_repo_hash',
                        required=True
    )
    parser.add_argument('--s3_bam_url',
                        required=False
    )
    parser.add_argument('--slurm_resource_core_count',
                        required=True
    )
    parser.add_argument('--slurm_resource_disk_gibibytes',
                        required=True
    )
    parser.add_argument('--slurm_resource_mem_mebibytes',
                        required=True
    )
    parser.add_argument('--status',
                        required=True
    )
    parser.add_argument('--table_name',
                        required=True
    )

    args = parser.parse_args()

    bam_signpost_id = args.bam_signpost_id
    hostname = args.hostname
    host_ipaddress = args.host_ipaddress
    host_mac = args.host_mac
    job_creation_uuid = args.job_creation_uuid
    job_path = args.job_path
    known_snp_signpost_id = args.known_snp_signpost_id
    num_threads = args.num_threads
    reference_fa_signpost_id = args.reference_fa_signpost_id
    runner_cwl_path = args.runner_cwl_path
    runner_repo_hash = args.runner_repo_hash
    run_uuid = args.run_uuid
    slurm_resource_core_count = args.slurm_resource_core_count
    slurm_resource_disk_gibibytes = args.slurm_resource_disk_gibibytes
    slurm_resource_mem_mebibytes = args.slurm_resource_mem_mebibytes
    status = args.status
    table_name = args.table_name

    if args.bam_uuid is not None:
        bam_uuid = args.bam_uuid
    else:
        bam_uuid = None

    if args.bam_uuid is not None:
        s3_bam_url = args.s3_bam_url
    else:
        s3_bam_url = None

    logger = setup_logging(args, run_uuid)

    sqlite_name = run_uuid + '.db'
    engine_path = 'sqlite:///' + sqlite_name
    engine = sqlalchemy.create_engine(engine_path, isolation_level='SERIALIZABLE')

    datetime_now = str(datetime.datetime.now())
    time_seconds = time.time()

    status_dict = dict()
    status_dict['bam_signpost_id'] = bam_signpost_id
    status_dict['bam_uuid'] = bam_uuid
    status_dict['datetime_now'] = datetime_now
    status_dict['hostname'] = hostname
    status_dict['host_ipaddress'] = host_ipaddress
    status_dict['host_mac'] = host_mac
    status_dict['job_creation_uuid'] = job_creation_uuid
    status_dict['job_path'] = job_path
    status_dict['known_snp_signpost_id'] = known_snp_signpost_id
    status_dict['num_threads'] = int(num_threads)
    status_dict['reference_fa_signpost_id'] = reference_fa_signpost_id
    status_dict['run_uuid'] = [run_uuid]
    status_dict['runner_cwl_path'] = runner_cwl_path
    status_dict['runner_repo_hash'] = runner_repo_hash
    status_dict['s3_bam_url'] = s3_bam_url
    status_dict['slurm_resource_core_count'] = slurm_resource_core_count
    status_dict['slurm_resource_disk_gibibytes'] = slurm_resource_disk_gibibytes
    status_dict['slurm_resource_mem_mebibytes'] = slurm_resource_mem_mebibytes
    status_dict['status'] = status
    status_dict['time_seconds'] = time_seconds

    df = pd.DataFrame(status_dict)
    df.to_sql(table_name, engine, if_exists='append')
    return

if __name__ == '__main__':
    main()
